@@ -5,7 +5,7 @@
  * ti-channel.c:
  * GObject wrapper for D-Bus method calls to org.freedesktop.Telepathy.Channel
  * 
- * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
+ * Copyright (C) 2006, 2007 INdT - Instituto Nokia de Tecnologia
  * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,17 +25,20 @@
 
 #include "ti-channel.h"
 #include "ti-signals-marshal.h"
+#include "ti-util.h"
 
 #include <dbus/dbus-glib.h>
 
 struct _TIChannelClass {
     GObjectClass parent;
-    
+
     guint closed_id; // "closed" signal id
     guint received_id; // "received" signal id
     guint sent_id; // "sent" signal id
     guint members_changed_id; // "members-changed" signal id
     guint group_flags_changed_id; // "group-flags-changed" signal id
+    guint got_rooms_id; // "got-rooms" signal id
+    guint listing_rooms_id; // "listing-rooms" signal id
 };
 
 G_DEFINE_TYPE (TIChannel, ti_channel, G_TYPE_OBJECT);
@@ -47,8 +50,12 @@ struct _TIChannelPrivate {
     gboolean disposed;
 
     DBusGProxy* dbus_proxy; // Proxy for main channel interface;
+    DBusGProxy* dbus_proxy_type_roomlist; // Proxy for Type.RoomList interface;
     DBusGProxy* dbus_proxy_type_text; // Proxy for Type.Text interface;
     DBusGProxy* dbus_proxy_group; // Proxy for Interface.Group interface;
+
+    gchar* service_name;
+    gchar* obj_path;
 };
 typedef struct _TIChannelPrivate TIChannelPrivate;
 
@@ -56,6 +63,8 @@ typedef struct _TIChannelPrivate TIChannelPrivate;
 
 // Function prototypes.
 static void _on_closed (gpointer proxy, TIChannel* channel);
+static void _on_got_rooms (gpointer proxy, GPtrArray* rooms, TIChannel* channel);
+static void _on_listing_rooms (gpointer proxy, gboolean listing, TIChannel* channel);
 static void _on_received (gpointer proxy,
                           guint id, guint timestamp, guint sender, guint type, guint flags, const gchar* text,
                           TIChannel* channel);
@@ -85,20 +94,10 @@ ti_channel_dispose (GObject *object)
         priv->disposed = TRUE;
     }
 
-    if (priv->dbus_proxy != NULL) {
-        g_object_unref (priv->dbus_proxy);
-        priv->dbus_proxy = NULL;
-    }
-
-    if (priv->dbus_proxy_type_text != NULL) {
-        g_object_unref (priv->dbus_proxy_type_text);
-        priv->dbus_proxy_type_text = NULL;
-    }
-
-    if (priv->dbus_proxy_group != NULL) {
-        g_object_unref (priv->dbus_proxy_group);
-        priv->dbus_proxy_group = NULL;
-    }
+    TI_OBJ_UNREF (priv->dbus_proxy);
+    TI_OBJ_UNREF (priv->dbus_proxy_type_roomlist);
+    TI_OBJ_UNREF (priv->dbus_proxy_type_text);
+    TI_OBJ_UNREF (priv->dbus_proxy_group);
 
     G_OBJECT_CLASS (ti_channel_parent_class)->dispose (object);
 }
@@ -109,10 +108,13 @@ ti_channel_dispose (GObject *object)
 static void
 ti_channel_finalize (GObject *object)
 {
-	//TIChannel* channel = TI_CHANNEL (object);
-    //TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE (channel);
+    TIChannel* channel = TI_CHANNEL (object);
+    TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE (channel);
 
-	G_OBJECT_CLASS (ti_channel_parent_class)->finalize (object);
+    g_free (priv->service_name);
+    g_free (priv->obj_path);
+
+    G_OBJECT_CLASS (ti_channel_parent_class)->finalize (object);
 }
 
 /**
@@ -121,17 +123,17 @@ ti_channel_finalize (GObject *object)
 static void
 ti_channel_class_init (TIChannelClass *ti_channel_class)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_channel_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (ti_channel_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_channel_dispose;
+    /* override base object methods */ 
+    gobject_class->dispose = ti_channel_dispose;
     gobject_class->finalize = ti_channel_finalize;
 
-	/* Add private */
-	g_type_class_add_private (ti_channel_class, sizeof (TIChannelPrivate));
-	
-	/* Signals */
-	
+    /* Add private */
+    g_type_class_add_private (ti_channel_class, sizeof (TIChannelPrivate));
+
+    /* Signals */
+
     ti_channel_class->closed_id =
         g_signal_new ("closed",
                       G_OBJECT_CLASS_TYPE (ti_channel_class),
@@ -181,6 +183,26 @@ ti_channel_class_init (TIChannelClass *ti_channel_class)
                       ti_marshal_VOID__UINT_UINT,
                       G_TYPE_NONE,
                       2, G_TYPE_UINT, G_TYPE_UINT);
+
+    ti_channel_class->got_rooms_id =
+        g_signal_new ("got-rooms",
+                      G_OBJECT_CLASS_TYPE (ti_channel_class),
+                      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                      0,
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOXED,
+                      G_TYPE_NONE,
+                      1, dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), G_TYPE_INVALID)));
+
+    ti_channel_class->listing_rooms_id =
+        g_signal_new ("listing-rooms",
+                      G_OBJECT_CLASS_TYPE (ti_channel_class),
+                      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                      0,
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE,
+                      1, G_TYPE_BOOLEAN);
 }
 
 /**
@@ -194,8 +216,12 @@ ti_channel_init (TIChannel *ti_channel)
     priv->disposed = FALSE;
 
     priv->dbus_proxy = NULL;
+    priv->dbus_proxy_type_roomlist = NULL;
     priv->dbus_proxy_type_text = NULL;
     priv->dbus_proxy_group = NULL;
+
+    priv->service_name = NULL;
+    priv->obj_path = NULL;
 }
 
 /**
@@ -218,12 +244,19 @@ ti_channel_new (const gchar* service_name, const gchar* obj_path)
         return NULL;
     }
 
-	channel = g_object_new (TI_TYPE_CHANNEL, NULL);
+    channel = g_object_new (TI_TYPE_CHANNEL, NULL);
     priv = TI_CHANNEL_GET_PRIVATE(channel);
 
+    priv->service_name = g_strdup (service_name);
+    priv->obj_path = g_strdup (obj_path);
+
     priv->dbus_proxy = dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
                                                   "org.freedesktop.Telepathy.Channel");
 
+    priv->dbus_proxy_type_roomlist =
+        dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
+                                   "org.freedesktop.Telepathy.Channel.Type.RoomList");
+
     priv->dbus_proxy_type_text =
         dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
                                    "org.freedesktop.Telepathy.Channel.Type.Text");
@@ -231,7 +264,7 @@ ti_channel_new (const gchar* service_name, const gchar* obj_path)
     priv->dbus_proxy_group =
         dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
                                    "org.freedesktop.Telepathy.Channel.Interface.Group");
-                                                  
+
     /* Tell DBus what the type signature of the signal callback is; this
      * allows us to sanity-check incoming messages before invoking the
      * callback.  You need to do this once for each proxy you create,
@@ -245,6 +278,13 @@ ti_channel_new (const gchar* service_name, const gchar* obj_path)
     dbus_g_proxy_add_signal (priv->dbus_proxy_type_text, "Received",
                              G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID);
 
+    // Type.RoomList interface signals 
+    dbus_g_proxy_add_signal (priv->dbus_proxy_type_roomlist, "GotRooms",
+                             dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), G_TYPE_INVALID)), G_TYPE_INVALID);
+
+    dbus_g_proxy_add_signal (priv->dbus_proxy_type_roomlist, "ListingRooms",
+                             G_TYPE_BOOLEAN, G_TYPE_INVALID);
+
     // Type.Text interface signals 
     dbus_g_proxy_add_signal (priv->dbus_proxy_type_text, "Sent",
                              G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID);
@@ -264,11 +304,18 @@ ti_channel_new (const gchar* service_name, const gchar* obj_path)
     // Main interface signals
     dbus_g_proxy_connect_signal (priv->dbus_proxy, "Closed", G_CALLBACK (_on_closed),
                                  channel, NULL);
-     
-    // Type.Text interface signals    
+
+    // Type.RoomList interface signals    
+    dbus_g_proxy_connect_signal (priv->dbus_proxy_type_roomlist, "GotRooms", G_CALLBACK (_on_got_rooms),
+                                 channel, NULL);
+
+    dbus_g_proxy_connect_signal (priv->dbus_proxy_type_roomlist, "ListingRooms", G_CALLBACK (_on_listing_rooms),
+                                 channel, NULL);
+
+    // Type.Text interface signals
     dbus_g_proxy_connect_signal (priv->dbus_proxy_type_text, "Received", G_CALLBACK (_on_received),
                                  channel, NULL);
-                                 
+
     dbus_g_proxy_connect_signal (priv->dbus_proxy_type_text, "Sent", G_CALLBACK (_on_sent),
                                  channel, NULL);
 
@@ -282,6 +329,28 @@ ti_channel_new (const gchar* service_name, const gchar* obj_path)
     return channel;
 }
 
+/*
+ * Get Service Name
+ */
+const gchar*
+ti_channel_get_service_name (TIChannel* channel)
+{
+    TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE(channel);
+
+    return priv->service_name;
+}
+
+/*
+ * Get Object Path
+ */
+const gchar*
+ti_channel_get_object_path (TIChannel* channel)
+{
+    TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE(channel);
+
+    return priv->obj_path;
+}
+
 /**
  * Get Interfaces
  */
@@ -628,8 +697,6 @@ _on_members_changed (gpointer proxy,
 {
     TIChannelClass* klass = TI_CHANNEL_GET_CLASS (channel);
 
-    g_debug ("ti-channel - _on_members_changed()");
-
     g_signal_emit (channel, klass->members_changed_id, 0, message, added, removed, local_pending, remote_pending, actor, reason);
 }
 
@@ -641,7 +708,71 @@ _on_group_flags_changed (gpointer proxy, guint added, guint removed, TIChannel*
 {
     TIChannelClass* klass = TI_CHANNEL_GET_CLASS (channel);
 
-    g_debug ("ti-channel - _on_group_flags_changed()");
-
     g_signal_emit (channel, klass->group_flags_changed_id, 0, added, removed);
 }
+
+/**
+ * ListRooms method from Type.RoomList interface
+ */
+void
+ti_channel_typeroomlist_list_rooms (TIChannel* channel, GError** error)
+{
+    TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE(channel);
+
+    *error = NULL;
+    if (!dbus_g_proxy_call (priv->dbus_proxy_type_roomlist, "ListRooms", error,
+                            G_TYPE_INVALID,
+                            G_TYPE_INVALID))
+    {
+        if (*error != NULL)
+        {
+            g_printerr ("Error: %s\n", (*error)->message);
+        }
+    }
+}
+
+
+/**
+ * GetListingRooms method from Type.RoomList interface
+ */
+gboolean
+ti_channel_typeroomlist_get_listing_rooms (TIChannel* channel, GError** error)
+{
+    TIChannelPrivate* priv = TI_CHANNEL_GET_PRIVATE(channel);
+    gboolean result;
+
+    *error = NULL;
+    if (!dbus_g_proxy_call (priv->dbus_proxy_type_roomlist, "GetListingRooms", error,
+                            G_TYPE_INVALID,
+                            G_TYPE_BOOLEAN, &result, G_TYPE_INVALID))
+    {
+        if (*error != NULL)
+        {
+            g_printerr ("Error: %s\n", (*error)->message);
+        }
+    }
+
+    return result;
+}
+
+/**
+ * On Got Rooms
+ */
+static void
+_on_got_rooms (gpointer proxy, GPtrArray* rooms, TIChannel* channel)
+{
+    TIChannelClass* klass = TI_CHANNEL_GET_CLASS (channel);
+
+    g_signal_emit (channel, klass->got_rooms_id, 0, rooms);
+}
+
+/**
+ * On Listing Rooms
+ */
+static void
+_on_listing_rooms (gpointer proxy, gboolean listing, TIChannel* channel)
+{
+    TIChannelClass* klass = TI_CHANNEL_GET_CLASS (channel);
+
+    g_signal_emit (channel, klass->listing_rooms_id, 0, listing);
+}
