@@ -5,7 +5,7 @@
  * ti-connection.h:
  * GObject wrapper for D-Bus method calls to org.freedesktop.Telepathy.Connection
  * 
- * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
+ * Copyright (C) 2006, 2007 INdT - Instituto Nokia de Tecnologia
  * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include "ti-connection.h"
 #include "ti-constants.h"
 #include "ti-signals-marshal.h"
+#include "ti-util.h"
 
 #include <dbus/dbus-glib.h>
 
@@ -36,12 +37,20 @@ struct _TIConnectionClass {
     // org.freedesktop.Telepathy.Connection.Interface.Aliasing - AliasesChanged signal
     guint aliases_changed_id;
 
+    // "capabilities-changed" signal id
+    // org.freedesktop.Telepathy.Connection.Interface.Capabilities - CapabilitiesChanged signal
+    guint capabilities_changed_id;
+
     guint status_changed_id;
-    
+
     // "presence-update" signal id
     // org.freedesktop.Telepathy.Connection.Interface.Presence - PresenceUpdate signal
     guint presence_update_id;
 
+    // "new-channel" signal id.
+    // org.freedesktop.Telepathy.Connection - NewChannel signal
+    guint new_channel_id;
+
     // GType for parameter "presence" on [...].Interface.Presence.PresenceUpdate(presence) signal
     GType g_type_presence_update;
 };
@@ -54,9 +63,11 @@ G_DEFINE_TYPE (TIConnection, ti_connection, G_TYPE_OBJECT);
 struct _TIConnectionPrivate {
     DBusGProxy* dbus_proxy; // Proxy for main interface
     DBusGProxy* dbus_proxy_aliasing; // Proxy for Aliasing interface
+    DBusGProxy* dbus_proxy_capabilities; // Proxy for Capabilities interface
     DBusGProxy* dbus_proxy_presence; // Proxy for Presence interface
-    
+
     gchar* service_name;
+    gchar* obj_path;
 };
 typedef struct _TIConnectionPrivate TIConnectionPrivate;
 
@@ -64,7 +75,9 @@ typedef struct _TIConnectionPrivate TIConnectionPrivate;
 
 // Function prototypes.
 static void _on_aliases_changed (gpointer proxy, GPtrArray* aliases, TIConnection* connection);
+static void _on_capabilities_changed (gpointer proxy, GPtrArray* caps, TIConnection* connection);
 static void _on_status_changed (gpointer proxy, guint status, guint reason, TIConnection* connection);
+static void _on_new_channel (gpointer proxy, const gchar* obj_path, const gchar* channel_type, guint handle_type, guint handle, gboolean suppress_handler, TIConnection* connection);
 static void _on_presence_update (gpointer proxy, GHashTable* presence, TIConnection* connection);
 
 /**
@@ -76,23 +89,10 @@ ti_connection_dispose (GObject *object)
     TIConnection* connection = TI_CONNECTION (object);
     TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE (connection);
 
-    if (priv->dbus_proxy != NULL)
-    {
-        g_object_unref (priv->dbus_proxy);
-        priv->dbus_proxy = NULL;
-    }
-
-    if (priv->dbus_proxy_aliasing != NULL)
-    {
-        g_object_unref (priv->dbus_proxy_aliasing);
-        priv->dbus_proxy_aliasing = NULL;
-    }
-
-    if (priv->dbus_proxy_presence != NULL)
-    {
-        g_object_unref (priv->dbus_proxy_presence);
-        priv->dbus_proxy_presence = NULL;
-    }
+    TI_OBJ_UNREF (priv->dbus_proxy);
+    TI_OBJ_UNREF (priv->dbus_proxy_aliasing);
+    TI_OBJ_UNREF (priv->dbus_proxy_capabilities);
+    TI_OBJ_UNREF (priv->dbus_proxy_presence);
 
     G_OBJECT_CLASS (ti_connection_parent_class)->dispose (object);
 }
@@ -103,12 +103,13 @@ ti_connection_dispose (GObject *object)
 static void
 ti_connection_finalize (GObject *object)
 {
-	TIConnection* connection = TI_CONNECTION (object);
+    TIConnection* connection = TI_CONNECTION (object);
     TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE (connection);
 
     g_free (priv->service_name);
+    g_free (priv->obj_path);
 
-	G_OBJECT_CLASS (ti_connection_parent_class)->finalize (object);
+    G_OBJECT_CLASS (ti_connection_parent_class)->finalize (object);
 }
 
 /**
@@ -117,18 +118,18 @@ ti_connection_finalize (GObject *object)
 static void
 ti_connection_class_init (TIConnectionClass *ti_connection_class)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_connection_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (ti_connection_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_connection_dispose;
+    /* override base object methods */
+    gobject_class->dispose = ti_connection_dispose;
     gobject_class->finalize = ti_connection_finalize;
 
     // A monster GType - a{u(ua{sa{sv}})}
     ti_connection_class->g_type_presence_update =
         (dbus_g_type_get_map ("GHashTable", G_TYPE_UINT, (dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)))), G_TYPE_INVALID))));
 
-	/* Add private */
-	g_type_class_add_private (ti_connection_class, sizeof (TIConnectionPrivate));
+    /* Add private */
+    g_type_class_add_private (ti_connection_class, sizeof (TIConnectionPrivate));
 
     ti_connection_class->aliases_changed_id =
         g_signal_new ("aliases-changed",
@@ -138,7 +139,17 @@ ti_connection_class_init (TIConnectionClass *ti_connection_class)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__BOXED,
                       G_TYPE_NONE,
-                      1, dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING)) );
+                      1, dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID)) );
+
+    ti_connection_class->capabilities_changed_id =
+        g_signal_new ("capabilities-changed",
+                      G_OBJECT_CLASS_TYPE (ti_connection_class),
+                      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                      0,
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOXED,
+                      G_TYPE_NONE,
+                      1, dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID)) );
 
     ti_connection_class->status_changed_id =
         g_signal_new ("status-changed",
@@ -159,6 +170,24 @@ ti_connection_class_init (TIConnectionClass *ti_connection_class)
                       g_cclosure_marshal_VOID__BOXED,
                       G_TYPE_NONE,
                       1, ti_connection_class->g_type_presence_update);
+
+    /**
+     * Signal:
+     * new-channel
+     *
+     * Parameters:
+     * OBJECT  - The TIChannel object
+     * BOOLEAN - Whether to suppress handlers
+     */
+    ti_connection_class->new_channel_id =
+        g_signal_new ("new-channel",
+                      G_OBJECT_CLASS_TYPE (ti_connection_class),
+                      G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                      0,
+                      NULL, NULL,
+                      ti_marshal_VOID__OBJECT_BOOLEAN,
+                      G_TYPE_NONE,
+                      2, G_TYPE_OBJECT, G_TYPE_BOOLEAN);
 }
 
 /**
@@ -171,7 +200,11 @@ ti_connection_init (TIConnection *ti_connection)
 
     priv->dbus_proxy = NULL;
     priv->dbus_proxy_aliasing = NULL;
+    priv->dbus_proxy_capabilities = NULL;
     priv->dbus_proxy_presence = NULL;
+
+    priv->service_name = NULL;
+    priv->obj_path = NULL;
 }
 
 /**
@@ -195,7 +228,7 @@ ti_connection_new (const gchar* service_name, const gchar* obj_path)
         return NULL;
     }
 
-	connection = g_object_new (TI_TYPE_CONNECTION, NULL);
+    connection = g_object_new (TI_TYPE_CONNECTION, NULL);
     priv = TI_CONNECTION_GET_PRIVATE (connection);
     klass = TI_CONNECTION_GET_CLASS (connection);
 
@@ -205,9 +238,13 @@ ti_connection_new (const gchar* service_name, const gchar* obj_path)
     priv->dbus_proxy_aliasing = dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
                                                            "org.freedesktop.Telepathy.Connection.Interface.Aliasing");
 
+    priv->dbus_proxy_capabilities = dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
+                                                           "org.freedesktop.Telepathy.Connection.Interface.Capabilities");
+
     priv->dbus_proxy_presence = dbus_g_proxy_new_for_name (dbus_conn, service_name, obj_path,
                                                            "org.freedesktop.Telepathy.Connection.Interface.Presence");
-    priv->service_name = g_strdup (service_name);                                                  
+    priv->service_name = g_strdup (service_name);
+    priv->obj_path = g_strdup (obj_path);
 
     /* Tell DBus what the type signature of the signal callback is; this
      * allows us to sanity-check incoming messages before invoking the
@@ -217,10 +254,15 @@ ti_connection_new (const gchar* service_name, const gchar* obj_path)
 
     dbus_g_proxy_add_signal (priv->dbus_proxy, "StatusChanged", G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID);
 
+    dbus_g_proxy_add_signal (priv->dbus_proxy, "NewChannel", DBUS_TYPE_G_OBJECT_PATH, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_BOOLEAN, G_TYPE_INVALID);
+
     dbus_g_proxy_add_signal (priv->dbus_proxy_presence, "PresenceUpdate", klass->g_type_presence_update, G_TYPE_INVALID);
 
     dbus_g_proxy_add_signal (priv->dbus_proxy_aliasing, "AliasesChanged",
-                             dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING)), G_TYPE_INVALID);
+                             dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID)), G_TYPE_INVALID);
+
+    dbus_g_proxy_add_signal (priv->dbus_proxy_capabilities, "CapabilitiesChanged",
+                             dbus_g_type_get_collection ("GPtrArray", dbus_g_type_get_struct ("GValueArray", G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID)), G_TYPE_INVALID);
 
     /* Actually connect to the signal.  Note you can call
      * dbus_g_proxy_connect_signal multiple times for one invocation of
@@ -230,12 +272,18 @@ ti_connection_new (const gchar* service_name, const gchar* obj_path)
     dbus_g_proxy_connect_signal (priv->dbus_proxy, "StatusChanged", G_CALLBACK (_on_status_changed),
                                  connection, NULL);
 
+    dbus_g_proxy_connect_signal (priv->dbus_proxy, "NewChannel", G_CALLBACK (_on_new_channel),
+                                 connection, NULL);
+
     dbus_g_proxy_connect_signal (priv->dbus_proxy_presence, "PresenceUpdate", G_CALLBACK (_on_presence_update),
                                  connection, NULL);
 
     dbus_g_proxy_connect_signal (priv->dbus_proxy_aliasing, "AliasesChanged", G_CALLBACK (_on_aliases_changed),
                                  connection, NULL);
 
+    dbus_g_proxy_connect_signal (priv->dbus_proxy_capabilities, "CapabilitiesChanged", G_CALLBACK (_on_capabilities_changed),
+                                 connection, NULL);
+
     return connection;
 }
 
@@ -250,6 +298,18 @@ ti_connection_get_service_name (TIConnection* connection)
     return priv->service_name;
 }
 
+/**
+ * Get Object Path
+ *
+ */
+const gchar*
+ti_connection_get_object_path (TIConnection* connection)
+{
+    TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE(connection);
+
+    return priv->obj_path;
+}
+
 /**
  * Get Interfaces
  */
@@ -378,6 +438,17 @@ _on_aliases_changed (gpointer proxy, GPtrArray* aliases, TIConnection* connectio
     g_signal_emit (connection, klass->aliases_changed_id, 0, aliases);
 }
 
+/**
+ * On Capabilities Changed
+ */
+static void
+_on_capabilities_changed (gpointer proxy, GPtrArray* caps, TIConnection* connection)
+{
+    TIConnectionClass* klass = TI_CONNECTION_GET_CLASS (connection);
+
+    g_signal_emit (connection, klass->capabilities_changed_id, 0, caps);
+}
+
 /**
  * On Status Changed
  */
@@ -389,6 +460,23 @@ _on_status_changed (gpointer proxy, guint status, guint reason, TIConnection* co
     g_signal_emit (connection, klass->status_changed_id, 0, status, reason);
 }
 
+/**
+ * On New Channel
+ */
+static void
+_on_new_channel (gpointer proxy, const gchar* obj_path, const gchar* channel_type, guint handle_type, guint handle, gboolean suppress_handler, TIConnection* connection)
+{
+    TIConnectionClass* klass = TI_CONNECTION_GET_CLASS (connection);
+    TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE(connection);
+    TIChannel* channel = NULL;
+
+    channel = ti_channel_new (priv->service_name, obj_path);
+
+    g_signal_emit (connection, klass->new_channel_id, 0, channel, suppress_handler);
+
+    TI_OBJ_UNREF (channel);
+}
+
 /**
  * On Presence Update
  */
@@ -554,9 +642,9 @@ ti_connection_request_channel (TIConnection* connection,
     {
         channel = ti_channel_new (priv->service_name, obj_path);
     }
-    
+
     g_free (obj_path);
-    
+
     return channel;
 }
 
@@ -781,3 +869,66 @@ ti_connection_ialiasing_get_alias_flags (TIConnection* connection, GError** erro
 
     return flags;
 }
+
+/**
+ * GetCapabilities method from Capabilities interface
+ */
+GPtrArray*
+ti_connection_icapabilities_get_capabilities (TIConnection* connection, GArray* handles, GError** error)
+{
+    TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE(connection);
+    static GType capability_gtype = G_TYPE_INVALID;
+    GPtrArray* capabilities = NULL;
+
+    if (capability_gtype == G_TYPE_INVALID)
+    {
+        capability_gtype = dbus_g_type_get_struct ("GValueArray",
+                                                  G_TYPE_UINT,
+                                                  G_TYPE_STRING,
+                                                  G_TYPE_UINT,
+                                                  G_TYPE_UINT,
+                                                  G_TYPE_INVALID);
+    }
+
+    *error = NULL;
+    if (!dbus_g_proxy_call (priv->dbus_proxy_capabilities, "GetCapabilities", error,
+                            DBUS_TYPE_G_UINT_ARRAY, handles, G_TYPE_INVALID,
+                            dbus_g_type_get_collection ("GPtrArray", capability_gtype), &capabilities, G_TYPE_INVALID))
+    {
+        if (*error != NULL)
+        {
+            g_printerr ("Error: %s\n", (*error)->message);
+        }
+    }
+
+    return capabilities;
+}
+
+/**
+ * AdvertiseCapabilities method from Capabilities interface
+ */
+void
+ti_connection_icapabilities_advertise_capabilities (TIConnection* connection, GPtrArray* add_caps, gchar** remove_caps, GError** error)
+{
+    TIConnectionPrivate* priv = TI_CONNECTION_GET_PRIVATE(connection);
+    static GType capability_gtype = G_TYPE_INVALID;
+
+    if (capability_gtype == G_TYPE_INVALID)
+    {
+        capability_gtype = dbus_g_type_get_struct ("GValueArray",
+                                                   G_TYPE_STRING,
+                                                   G_TYPE_UINT,
+                                                   G_TYPE_INVALID);
+    }
+
+    *error = NULL;
+    if (!dbus_g_proxy_call (priv->dbus_proxy_capabilities, "AdvertiseCapabilities", error,
+                            dbus_g_type_get_collection ("GPtrArray", capability_gtype), add_caps, G_TYPE_STRV, remove_caps, G_TYPE_INVALID,
+                            G_TYPE_INVALID))
+    {
+        if (*error != NULL)
+        {
+            g_printerr ("Error: %s\n", (*error)->message);
+        }
+    }
+}
