@@ -80,8 +80,8 @@ static void _ti_dlg_set_aliases_build_contacts_treeview (TIDlgSetAliases* self);
 static void _ti_dlg_set_aliases_setup (TIDlgSetAliases* self);
 static GHashTable* _ti_dlg_set_aliases_get_aliases_hash_table (TIDlgSetAliases* self);
 static void _ti_dlg_set_aliases_add (TIDlgSetAliases* self);
-static void _ti_dlg_set_aliases_contact_handle_display_mode_changed (TIDlgSetAliases* self, guint contact_handle_display_mode);
-static void _ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self, guint contact_handle_display_mode);
+static void _ti_dlg_set_aliases_handle_display_mode_changed (TIDlgSetAliases* self, guint handle_display_mode);
+static void _ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self, guint handle_display_mode);
 
 /**
  * Drop all references to other objects.
@@ -94,9 +94,9 @@ ti_dlg_set_aliases_dispose (GObject *object)
 
     if (priv->preferences != NULL)
     {
-        g_signal_handlers_disconnect_by_func(priv->preferences,
-                                             G_CALLBACK (_ti_dlg_set_aliases_contact_handle_display_mode_changed),
-                                             self);
+        g_signal_handlers_disconnect_by_func (priv->preferences,
+                                              G_CALLBACK (_ti_dlg_set_aliases_handle_display_mode_changed),
+                                              self);
 
         g_object_unref (priv->preferences);
         priv->preferences = NULL;
@@ -203,8 +203,8 @@ _ti_dlg_set_aliases_setup (TIDlgSetAliases* self)
     TIDlgSetAliasesPrivate *priv = TI_DLG_SET_ALIASES_GET_PRIVATE (self);
     GtkWidget* widget;
 
-    g_signal_connect_swapped (priv->preferences, "contact-handle-display-changed",
-                              G_CALLBACK (_ti_dlg_set_aliases_contact_handle_display_mode_changed), self);
+    g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+                              G_CALLBACK (_ti_dlg_set_aliases_handle_display_mode_changed), self);
 
     // The dialog itself
     priv->dialog = glade_xml_get_widget (priv->glade_xml, "dialog_set_aliases");
@@ -252,7 +252,7 @@ _ti_dlg_set_aliases_setup (TIDlgSetAliases* self)
     g_assert (GTK_IS_ENTRY (priv->handle_entry));
 
     // Set to current mode
-    _ti_dlg_set_aliases_contact_handle_display_mode_changed (self, ti_preferences_get_contact_handle_display_mode (priv->preferences));
+    _ti_dlg_set_aliases_handle_display_mode_changed (self, ti_preferences_get_handle_display_mode (priv->preferences));
 }
 
 /**
@@ -314,8 +314,8 @@ _ti_dlg_set_aliases_build_aliases_treeview (TIDlgSetAliases* self)
 
     renderer = gtk_cell_renderer_text_new ();
 
-    if (ti_preferences_get_contact_handle_display_mode (priv->preferences) ==
-        TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (ti_preferences_get_handle_display_mode (priv->preferences) ==
+        TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         /* Handle column */
         column = gtk_tree_view_column_new_with_attributes ("Handle",
@@ -333,14 +333,14 @@ _ti_dlg_set_aliases_build_aliases_treeview (TIDlgSetAliases* self)
                                                            NULL);
         gtk_tree_view_append_column (priv->aliases_treeview, column);
     }
-    
+
     /* Alias column */
     column = gtk_tree_view_column_new_with_attributes ("Alias",
                                                        renderer,
                                                        "text", TI_ALIASES_COLUMN_ALIAS,
                                                        NULL);
     gtk_tree_view_append_column (priv->aliases_treeview, column);
-   
+
     priv->aliases_selection = gtk_tree_view_get_selection (priv->aliases_treeview);
 
     gtk_tree_selection_set_mode (priv->aliases_selection, GTK_SELECTION_MULTIPLE);
@@ -422,11 +422,11 @@ _ti_dlg_set_aliases_add (TIDlgSetAliases* self)
     guint handle;
     gchar* name;
     const gchar* alias;
-    guint handle_display_mode = ti_preferences_get_contact_handle_display_mode (priv->preferences);
+    guint handle_display_mode = ti_preferences_get_handle_display_mode (priv->preferences);
     int result;
 
     // Get selected contact
-    if (handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         result = sscanf (gtk_entry_get_text (priv->handle_entry), "%u", &handle);
         if (result != 1)
@@ -464,10 +464,10 @@ _ti_dlg_set_aliases_add (TIDlgSetAliases* self)
 }
 
 /**
- * Contact Handle Display Mode Changed
+ * Handle Display Mode Changed
  */
 static void
-_ti_dlg_set_aliases_contact_handle_display_mode_changed (TIDlgSetAliases* self, guint contact_handle_display_mode)
+_ti_dlg_set_aliases_handle_display_mode_changed (TIDlgSetAliases* self, guint handle_display_mode)
 {
     TIDlgSetAliasesPrivate* priv = TI_DLG_SET_ALIASES_GET_PRIVATE (self);
 
@@ -476,7 +476,7 @@ _ti_dlg_set_aliases_contact_handle_display_mode_changed (TIDlgSetAliases* self,
     gtk_container_remove (GTK_CONTAINER (priv->main_box), priv->handle_box);
 
     // Add one according to the global preferences
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_box_pack_start (priv->main_box, priv->handle_box, FALSE, TRUE, 0);
         gtk_box_reorder_child (priv->main_box, priv->handle_box, 0);
@@ -487,14 +487,14 @@ _ti_dlg_set_aliases_contact_handle_display_mode_changed (TIDlgSetAliases* self,
         gtk_box_reorder_child (priv->main_box, GTK_WIDGET (priv->contacts_scrolledwnd), 0);
     }
 
-    _ti_dlg_set_aliases_change_aliases_treeview_display_mode (self, contact_handle_display_mode);
+    _ti_dlg_set_aliases_change_aliases_treeview_display_mode (self, handle_display_mode);
 }
 
 /**
  * Change Aliases Treeview Display Mode
  */
 static void
-_ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self, guint contact_handle_display_mode)
+_ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self, guint handle_display_mode)
 {
     TIDlgSetAliasesPrivate* priv = TI_DLG_SET_ALIASES_GET_PRIVATE (self);
     GtkTreeViewColumn* contact_handle_column;
@@ -508,7 +508,7 @@ _ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self,
 
     renderer = GTK_CELL_RENDERER (renderers_list->data);
 
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Handle");
 
@@ -516,7 +516,7 @@ _ti_dlg_set_aliases_change_aliases_treeview_display_mode (TIDlgSetAliases* self,
                                              "text", TI_ALIASES_COLUMN_HANDLE,
                                              NULL);
     }
-    else // TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_NAME
+    else // TI_PREFERENCES_HANDLE_DISPLAY_NAME
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Name");
 
