@@ -89,8 +89,8 @@ static void _ti_handle_list_editor_update_button_add_sensitivity (TIHandleListEd
 static void _ti_handle_list_editor_update_button_remove_sensitivity (TIHandleListEditor* self);
 static void _ti_handle_list_editor_name_ui_add (TIHandleListEditor* self);
 static void _ti_handle_list_editor_name_ui_update_button_add_sensitivity (TIHandleListEditor* self);
-static void _ti_handle_list_editor_contact_handle_display_mode_changed (TIHandleListEditor* self, guint contact_handle_display_mode);
-static void _ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self, guint contact_handle_display_mode);
+static void _ti_handle_list_editor_handle_display_mode_changed (TIHandleListEditor* self, guint handle_display_mode);
+static void _ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self, guint handle_display_mode);
 
 /**
  * Drop all references to other objects.
@@ -98,10 +98,15 @@ static void _ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEd
 static void
 ti_handle_list_editor_dispose (GObject *object)
 {
-    TIHandleListEditorPrivate *priv = TI_HANDLE_LIST_EDITOR_GET_PRIVATE (object);
+    TIHandleListEditor* self  = TI_HANDLE_LIST_EDITOR (object);
+    TIHandleListEditorPrivate* priv = TI_HANDLE_LIST_EDITOR_GET_PRIVATE (self);
 
     if (priv->preferences != NULL)
     {
+        g_signal_handlers_disconnect_by_func(priv->preferences,
+                                             G_CALLBACK (_ti_handle_list_editor_handle_display_mode_changed),
+                                             self);
+
         g_object_unref (priv->preferences);
         priv->preferences = NULL;
     }
@@ -133,13 +138,13 @@ ti_handle_list_editor_dispose (GObject *object)
 static void
 ti_handle_list_editor_class_init (TIHandleListEditorClass* handle_list_editor_class)
 {
-	GObjectClass* gobject_class = G_OBJECT_CLASS (handle_list_editor_class);
+    GObjectClass* gobject_class = G_OBJECT_CLASS (handle_list_editor_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_handle_list_editor_dispose;
-		
-	/* Add private */
-	g_type_class_add_private (handle_list_editor_class, sizeof (TIHandleListEditorPrivate));
+    /* override base object methods */ 
+    gobject_class->dispose = ti_handle_list_editor_dispose;
+
+    /* Add private */
+    g_type_class_add_private (handle_list_editor_class, sizeof (TIHandleListEditorPrivate));
 }
 
 /**
@@ -177,8 +182,8 @@ ti_handle_list_editor_new (TIHandleMapper* handle_mapper)
     _ti_handle_list_editor_init_name_ui (handle_list_editor);
     _ti_handle_list_editor_build_tree_view (handle_list_editor);
 
-    g_signal_connect_swapped (priv->preferences, "contact-handle-display-changed",
-                              G_CALLBACK (_ti_handle_list_editor_contact_handle_display_mode_changed), handle_list_editor);
+    g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+                              G_CALLBACK (_ti_handle_list_editor_handle_display_mode_changed), handle_list_editor);
 
     // Button Remove
     priv->button_remove = GTK_BUTTON (gtk_button_new_from_stock (GTK_STOCK_REMOVE));
@@ -205,8 +210,8 @@ ti_handle_list_editor_new (TIHandleMapper* handle_mapper)
     // Widget Root
     gtk_box_set_homogeneous (GTK_BOX (handle_list_editor), TRUE);
 
-    if (ti_preferences_get_contact_handle_display_mode (priv->preferences) ==
-        TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (ti_preferences_get_handle_display_mode (priv->preferences) ==
+        TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_box_pack_start (GTK_BOX (handle_list_editor), priv->number_ui.vbox_buttons_and_entry, FALSE, TRUE, 0);
     }
@@ -327,8 +332,8 @@ _ti_handle_list_editor_build_tree_view (TIHandleListEditor* self)
 
     renderer = gtk_cell_renderer_text_new ();
 
-    if (ti_preferences_get_contact_handle_display_mode (priv->preferences) ==
-        TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (ti_preferences_get_handle_display_mode (priv->preferences) ==
+        TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         column = gtk_tree_view_column_new_with_attributes ("Handle",
                                                            renderer,
@@ -527,10 +532,10 @@ _ti_handle_list_editor_name_ui_update_button_add_sensitivity (TIHandleListEditor
 }
 
 /**
- * Contact Handle Display Mode Changed
+ * Handle Display Mode Changed
  */
 static void
-_ti_handle_list_editor_contact_handle_display_mode_changed (TIHandleListEditor* self, guint contact_handle_display_mode)
+_ti_handle_list_editor_handle_display_mode_changed (TIHandleListEditor* self, guint handle_display_mode)
 {
     TIHandleListEditorPrivate *priv = TI_HANDLE_LIST_EDITOR_GET_PRIVATE (self);
 
@@ -539,7 +544,7 @@ _ti_handle_list_editor_contact_handle_display_mode_changed (TIHandleListEditor*
     gtk_container_remove (GTK_CONTAINER (self), priv->number_ui.vbox_buttons_and_entry);
 
     // Add one according to the global preferences
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_box_pack_start (GTK_BOX (self), priv->number_ui.vbox_buttons_and_entry, TRUE, TRUE, 0);
         gtk_box_reorder_child (GTK_BOX (self), priv->number_ui.vbox_buttons_and_entry, 0);
@@ -550,14 +555,14 @@ _ti_handle_list_editor_contact_handle_display_mode_changed (TIHandleListEditor*
         gtk_box_reorder_child (GTK_BOX (self), priv->name_ui.vbox_buttons_and_list, 0);
     }
 
-    _ti_handle_list_editor_change_tree_view_display_mode (self, contact_handle_display_mode);
+    _ti_handle_list_editor_change_tree_view_display_mode (self, handle_display_mode);
 }
 
 /**
  * Change Tree View Display Mode
  */
 static void
-_ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self, guint contact_handle_display_mode)
+_ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self, guint handle_display_mode)
 {
     TIHandleListEditorPrivate *priv = TI_HANDLE_LIST_EDITOR_GET_PRIVATE (self);
     GtkTreeViewColumn* contact_handle_column;
@@ -571,7 +576,7 @@ _ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self,
 
     renderer = GTK_CELL_RENDERER (renderers_list->data);
 
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Handle");
 
@@ -579,7 +584,7 @@ _ti_handle_list_editor_change_tree_view_display_mode (TIHandleListEditor* self,
                                              "text", TI_COLUMN_HANDLES_LIST_HANDLE,
                                              NULL);
     }
-    else // TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_NAME
+    else // TI_PREFERENCES_HANDLE_DISPLAY_NAME
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Name");
 
