@@ -81,14 +81,13 @@ static void _ti_page_channel_group_log_group_flags_changed (TIPageChannelGroup*
 static void _ti_page_channel_group_refresh_group_flags_checkboxes (TIPageChannelGroup* page_channel_group);
 static void _ti_page_channel_group_update_selection_permissions (TIPageChannelGroup* page_channel_group);
 static void _group_flags_to_string (GString* text, guint flags);
-static void _ti_page_channel_group_contact_handle_display_mode_changed (TIPageChannelGroup* page_channel_group, guint contact_handle_display_mode);
+static void _ti_page_channel_group_handle_display_mode_changed (TIPageChannelGroup* page_channel_group, guint handle_display_mode);
+static gboolean _ti_page_channel_get_contact_row (TIPageChannelGroup* page_channel_group, guint contact_handle, GtkTreeIter* iter);
 
 /**
  * Instance private data.
  */
 struct _TIPageChannelGroupPrivate {
-    gboolean disposed;
-
     TIChannel* channel;
 
     TIPreferences* preferences;
@@ -102,7 +101,7 @@ struct _TIPageChannelGroupPrivate {
     GtkListStore* members_list;
     GtkTreeView* members_tree_view;
     GtkTreeSelection* members_selection;
-    
+
     GtkWidget* checkbutton_can_add;
     GtkWidget* checkbutton_can_remove;
     GtkWidget* checkbutton_can_rescind;
@@ -112,7 +111,7 @@ struct _TIPageChannelGroupPrivate {
     GtkWidget* checkbutton_message_reject;
     GtkWidget* checkbutton_message_rescind;
     GtkWidget* checkbutton_channel_specific_handles;
-    
+
     GtkButton* button_remove_members;
     GtkButton* button_add_members;
 
@@ -131,15 +130,6 @@ ti_page_channel_group_dispose (GObject *object)
     TIPageChannelGroup *page_channel_group = TI_PAGE_CHANNEL_GROUP (object);
     TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (page_channel_group);
 
-    if (priv->disposed)
-    {
-        return;
-    }
-    else
-    {
-        priv->disposed = TRUE;
-    }
-
     if (priv->channel != NULL)
     {
         g_signal_handlers_disconnect_by_func(priv->channel,
@@ -147,7 +137,7 @@ ti_page_channel_group_dispose (GObject *object)
                                              page_channel_group);
 
         g_signal_handlers_disconnect_by_func(priv->channel,
-                                             G_CALLBACK (_ti_page_channel_group_members_changed),
+                                             G_CALLBACK (_ti_page_channel_group_group_flags_changed),
                                              page_channel_group);
 
         g_object_unref (priv->channel);
@@ -157,36 +147,17 @@ ti_page_channel_group_dispose (GObject *object)
     if (priv->preferences != NULL)
     {
         g_signal_handlers_disconnect_by_func(priv->preferences,
-                                             G_CALLBACK (_ti_page_channel_group_contact_handle_display_mode_changed),
+                                             G_CALLBACK (_ti_page_channel_group_handle_display_mode_changed),
                                              page_channel_group);
 
         g_object_unref (priv->preferences);
         priv->preferences = NULL;
     }
 
-    if (priv->handle_mapper != NULL)
-    {
-        g_object_unref (priv->handle_mapper);
-        priv->handle_mapper = NULL;
-    }
-
-    if (priv->dialog_add_members != NULL)
-    {
-        g_object_unref (priv->dialog_add_members);
-        priv->dialog_add_members = NULL;
-    }
-    
-    if (priv->dialog_get_text != NULL)
-    {
-        g_object_unref (priv->dialog_get_text);
-        priv->dialog_get_text = NULL;
-    }
-    
-    if (priv->members_list != NULL)
-    {
-        g_object_unref (priv->members_list);
-        priv->members_list = NULL;
-    }
+    TI_OBJ_UNREF (priv->handle_mapper);
+    TI_OBJ_UNREF (priv->dialog_add_members);
+    TI_OBJ_UNREF (priv->dialog_get_text);
+    TI_OBJ_UNREF (priv->members_list);
 
     G_OBJECT_CLASS (ti_page_channel_group_parent_class)->dispose (object);
 }
@@ -197,16 +168,16 @@ ti_page_channel_group_dispose (GObject *object)
 static void
 ti_page_channel_group_class_init (TIPageChannelGroupClass *ti_page_channel_group_class)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_page_channel_group_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (ti_page_channel_group_class);
     TIPageClass* page_class = TI_PAGE_CLASS (ti_page_channel_group_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_page_channel_group_dispose;
+    /* override base object methods */ 
+    gobject_class->dispose = ti_page_channel_group_dispose;
 
     page_class->setup_page = _ti_page_channel_group_setup_page;
 
-	/* Add private */
-	g_type_class_add_private (ti_page_channel_group_class, sizeof (TIPageChannelGroupPrivate));
+    /* Add private */
+    g_type_class_add_private (ti_page_channel_group_class, sizeof (TIPageChannelGroupPrivate));
 }
 
 /**
@@ -217,7 +188,6 @@ ti_page_channel_group_init (TIPageChannelGroup *ti_page_channel_group)
 {
     TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (ti_page_channel_group);
 
-    priv->disposed = FALSE;
     priv->channel = NULL;
     priv->preferences = ti_preferences_new ();
     priv->handle_mapper = NULL;
@@ -249,8 +219,8 @@ ti_page_channel_group_new (GtkNotebook* parent_notebook, GtkWindow* parent_windo
     g_signal_connect_swapped (channel, "members-changed", G_CALLBACK (_ti_page_channel_group_members_changed), page_channel_group);
     g_signal_connect_swapped (channel, "group-flags-changed", G_CALLBACK (_ti_page_channel_group_group_flags_changed), page_channel_group);
 
-    g_signal_connect_swapped (priv->preferences, "contact-handle-display-changed",
-                              G_CALLBACK (_ti_page_channel_group_contact_handle_display_mode_changed), page_channel_group);
+    g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+                              G_CALLBACK (_ti_page_channel_group_handle_display_mode_changed), page_channel_group);
 
     priv->dialog_add_members = ti_dlg_add_members_new (parent_window, handle_mapper);
     if (priv->dialog_add_members == NULL)
@@ -339,7 +309,7 @@ _ti_page_channel_group_build_members_treeview (TIPageChannelGroup* page_channel_
 
     renderer = gtk_cell_renderer_text_new ();
 
-    if (ti_preferences_get_contact_handle_display_mode (priv->preferences) == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (ti_preferences_get_handle_display_mode (priv->preferences) == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         column = gtk_tree_view_column_new_with_attributes ("Handle",
                                                            renderer,
@@ -449,8 +419,13 @@ _ti_page_channel_group_append_members_to_list (TIPageChannelGroup* page_channel_
         {
             name = g_strdup_printf ("%u", handle);
         }
-        
-        gtk_list_store_append (priv->members_list, &iter);
+
+        // If there's not already a row for that contact, we create one.
+        if (!_ti_page_channel_get_contact_row (page_channel_group, handle, &iter))
+        {
+            gtk_list_store_append (priv->members_list, &iter);
+        }
+
         gtk_list_store_set (priv->members_list, &iter,
                             TI_COLUMN_MEMBERS_LIST_HANDLE, handle,
                             TI_COLUMN_MEMBERS_LIST_NAME, name,
@@ -939,10 +914,10 @@ _group_flags_to_string (GString* text, guint flags)
 }
 
 /**
- * Contact Handle Display Mode Changed
+ * Handle Display Mode Changed
  */
 static void
-_ti_page_channel_group_contact_handle_display_mode_changed (TIPageChannelGroup* page_channel_group, guint contact_handle_display_mode)
+_ti_page_channel_group_handle_display_mode_changed (TIPageChannelGroup* page_channel_group, guint handle_display_mode)
 {
     TIPageChannelGroupPrivate *priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (page_channel_group);
     GtkTreeViewColumn* contact_handle_column;
@@ -958,7 +933,7 @@ _ti_page_channel_group_contact_handle_display_mode_changed (TIPageChannelGroup*
 
     renderer = GTK_CELL_RENDERER (renderers_list->data);
 
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Handle");
 
@@ -966,7 +941,7 @@ _ti_page_channel_group_contact_handle_display_mode_changed (TIPageChannelGroup*
                                              "text", TI_COLUMN_MEMBERS_LIST_HANDLE,
                                              NULL);
     }
-    else // TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_NAME
+    else // TI_PREFERENCES_HANDLE_DISPLAY_NAME
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Name");
 
@@ -978,3 +953,34 @@ _ti_page_channel_group_contact_handle_display_mode_changed (TIPageChannelGroup*
     // Clean up
     g_list_free (renderers_list);
 }
+
+/**
+ * Get Contact Row
+ */
+static gboolean
+_ti_page_channel_get_contact_row (TIPageChannelGroup* page_channel_group, guint contact_handle, GtkTreeIter* iter)
+{
+    TIPageChannelGroupPrivate* priv = TI_PAGE_CHANNEL_GROUP_GET_PRIVATE (page_channel_group);
+    gboolean ok;
+    gboolean found = FALSE;
+    guint curr_handle;
+
+    ok = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->members_list), iter);
+    while (ok && !found)
+    {
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->members_list), iter,
+                            TI_COLUMN_MEMBERS_LIST_HANDLE, &curr_handle,
+                            -1);
+
+        if (curr_handle == contact_handle)
+        {
+            found = TRUE;
+        }
+        else
+        {
+            ok = gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->members_list), iter);
+        }
+    }
+
+    return found;
+}
