@@ -74,14 +74,12 @@ static void _ti_page_presence_clear_presence_list (TIPagePresence* page_presence
 static void _ti_page_presence_add_user_status (TIPagePresence* page_presence);
 static void _ti_page_presence_clear_user_status (TIPagePresence* page_presence);
 static void _ti_page_presence_remove_user_status (TIPagePresence* page_presence);
-static void _ti_page_presence_contact_handle_display_mode_changed (TIPagePresence* page_presence, guint contact_handle_display_mode);
+static void _ti_page_presence_handle_display_mode_changed (TIPagePresence* page_presence, guint handle_display_mode);
 
 /**
  * Instance private data.
  */
 struct _TIPagePresencePrivate {
-    gboolean disposed;
-
     TIPreferences* preferences;
     TIHandleMapper* handle_mapper;
 
@@ -118,43 +116,22 @@ typedef struct _TIPagePresencePrivate TIPagePresencePrivate;
 static void
 ti_page_presence_dispose (GObject *object)
 {
-    TIPagePresence *page_presence = TI_PAGE_PRESENCE (object);
-    TIPagePresencePrivate *priv = TI_PAGE_PRESENCE_GET_PRIVATE (page_presence);
-
-    if (priv->disposed)
-        return;
-    else
-        priv->disposed = TRUE;
+    TIPagePresence* self = TI_PAGE_PRESENCE (object);
+    TIPagePresencePrivate *priv = TI_PAGE_PRESENCE_GET_PRIVATE (self);
 
     if (priv->preferences != NULL)
     {
+        g_signal_handlers_disconnect_by_func(priv->preferences,
+                                             G_CALLBACK (_ti_page_presence_handle_display_mode_changed),
+                                             self);
         g_object_unref (priv->preferences);
         priv->preferences = NULL;
     }
 
-    if (priv->handle_mapper != NULL)
-    {
-        g_object_unref (priv->handle_mapper);
-        priv->handle_mapper = NULL;
-    }
-
-    if (priv->connection != NULL)
-    {
-        g_object_unref (priv->connection);
-        priv->connection = NULL;
-    }
-
-    if (priv->dlg_user_status != NULL)
-    {
-        g_object_unref (priv->dlg_user_status);
-        priv->dlg_user_status = NULL;
-    }
-
-    if (priv->dlg_handles_list != NULL)
-    {
-        g_object_unref (priv->dlg_handles_list);
-        priv->dlg_handles_list = NULL;
-    }
+    TI_OBJ_UNREF (priv->handle_mapper);
+    TI_OBJ_UNREF (priv->connection);
+    TI_OBJ_UNREF (priv->dlg_user_status);
+    TI_OBJ_UNREF (priv->dlg_handles_list);
 
     if (priv->hashtable_valid_status_opt_params_list != NULL)
     {
@@ -198,8 +175,6 @@ ti_page_presence_init (TIPagePresence *ti_page_presence)
 {
     TIPagePresencePrivate *priv = TI_PAGE_PRESENCE_GET_PRIVATE (ti_page_presence);
 
-    priv->disposed = FALSE;
-
     priv->preferences = ti_preferences_new ();
     priv->handle_mapper = NULL;
     priv->connection = NULL;
@@ -220,7 +195,7 @@ ti_page_presence_new (GtkWindow* parent_wnd, GtkNotebook* parent_notebook, TICon
 
     g_assert (parent_wnd != NULL);
 
-	page_presence = g_object_new (TI_TYPE_PAGE_PRESENCE, NULL);
+    page_presence = g_object_new (TI_TYPE_PAGE_PRESENCE, NULL);
 
     priv = TI_PAGE_PRESENCE_GET_PRIVATE (page_presence);
     priv->parent_wnd = parent_wnd;
@@ -234,8 +209,8 @@ ti_page_presence_new (GtkWindow* parent_wnd, GtkNotebook* parent_notebook, TICon
     priv->dlg_handles_list = ti_dlg_handles_list_new (parent_wnd, handle_mapper, "Request Presence");
     g_assert (priv->dlg_handles_list != NULL);
 
-    g_signal_connect_swapped (priv->preferences, "contact-handle-display-changed",
-                              G_CALLBACK (_ti_page_presence_contact_handle_display_mode_changed), page_presence);
+    g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+                              G_CALLBACK (_ti_page_presence_handle_display_mode_changed), page_presence);
 
     _ti_page_new ((TIPage**)&page_presence, parent_notebook, "page-presence.xml");
 
@@ -424,8 +399,8 @@ _ti_page_presence_build_presence_treeview (TIPagePresence* page_presence, GladeX
 
     renderer = gtk_cell_renderer_text_new ();
 
-    if (ti_preferences_get_contact_handle_display_mode (priv->preferences) ==
-        TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (ti_preferences_get_handle_display_mode (priv->preferences) ==
+        TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         /* Handle column */
         column = gtk_tree_view_column_new_with_attributes ("Handle",
@@ -443,7 +418,7 @@ _ti_page_presence_build_presence_treeview (TIPagePresence* page_presence, GladeX
                                                            NULL);
         gtk_tree_view_append_column (priv->presence_treeview, column);
     }
-    
+
     /* Timestamp column */
     column = gtk_tree_view_column_new_with_attributes ("Timestamp",
                                                        renderer,
@@ -457,7 +432,7 @@ _ti_page_presence_build_presence_treeview (TIPagePresence* page_presence, GladeX
                                                        "text", TI_COLUMN_PRESENCE_LIST_STATUS,
                                                        NULL);
     gtk_tree_view_append_column (priv->presence_treeview, column);    
-   
+
     priv->presence_selection = gtk_tree_view_get_selection (priv->presence_treeview);
 
     g_signal_connect_swapped (priv->presence_selection, "changed",
@@ -1067,10 +1042,10 @@ _ti_page_presence_remove_user_status (TIPagePresence* page_presence)
 }
 
 /**
- * Contact Handle Display Mode Changed
+ * Handle Display Mode Changed
  */
 static void
-_ti_page_presence_contact_handle_display_mode_changed (TIPagePresence* page_presence, guint contact_handle_display_mode)
+_ti_page_presence_handle_display_mode_changed (TIPagePresence* page_presence, guint handle_display_mode)
 {
     TIPagePresencePrivate* priv = TI_PAGE_PRESENCE_GET_PRIVATE (page_presence);
     GtkTreeViewColumn* contact_handle_column;
@@ -1084,7 +1059,7 @@ _ti_page_presence_contact_handle_display_mode_changed (TIPagePresence* page_pres
 
     renderer = GTK_CELL_RENDERER (renderers_list->data);
 
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Handle");
 
@@ -1092,7 +1067,7 @@ _ti_page_presence_contact_handle_display_mode_changed (TIPagePresence* page_pres
                                              "text", TI_COLUMN_PRESENCE_LIST_HANDLE,
                                              NULL);
     }
-    else // TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_NAME
+    else // TI_PREFERENCES_HANDLE_DISPLAY_NAME
     {
         gtk_tree_view_column_set_title (contact_handle_column, "Name");
 
