@@ -5,7 +5,7 @@
  * ti-wnd-channel.c:
  * Channel Window - UI to expose org.freedesktop.Telepathy.Channel functionality
  * 
- * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
+ * Copyright (C) 2006, 2007 INdT - Instituto Nokia de Tecnologia
  * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,12 @@
 #include "ti-wnd-channel.h"
 #include "ti-page-channel-group.h"
 #include "ti-page-channel-text.h"
+#include "ti-page-properties.h"
+#include "ti-page-room-list.h"
 #include "ti-config.h"
 #include "ti-preferences.h"
 #include "ti-constants.h"
+#include "ti-util.h"
 
 #include <glade/glade.h>
 #include <string.h>
@@ -50,16 +53,14 @@ static void _ti_wnd_channel_close_channel (TIWndChannel* wnd_channel);
 static void _ti_wnd_channel_add_interface_pages (TIWndChannel* wnd_channel);
 static void _ti_wnd_channel_add_interface_page (TIWndChannel* wnd_channel, gchar* interface_name);
 static gchar* _get_handle_type_str (guint handle_type);
-static void _ti_wnd_channel_channel_closed (TIWndChannel* wnd_channel);
+static void _ti_wnd_channel_close_window (TIWndChannel* wnd_channel);
 static void _ti_wnd_channel_setup_type_interface_page (TIWndChannel* wnd_channel);
-static void _ti_wnd_channel_contact_handle_display_mode_changed (TIWndChannel* wnd_channel, guint contact_handle_display_mode);
+static void _ti_wnd_channel_handle_display_mode_changed (TIWndChannel* wnd_channel, guint handle_display_mode);
 
 /**
  * Instance private data.
  */
 struct _TIWndChannelPrivate {
-    gboolean disposed;
-
     TIHandleMapper* handle_mapper;
     TIPreferences* preferences;
 
@@ -80,9 +81,13 @@ struct _TIWndChannelPrivate {
     GtkListStore* interfaces_list;
 
     TIChannel* channel;
-    
+
     TIPageChannelGroup* page_group;
+    TIPageRoomList* page_room_list;
     TIPageChannelText* page_text;
+    TIPageProperties* page_properties;
+
+    gboolean should_close_window;
 };
 typedef struct _TIWndChannelPrivate TIWndChannelPrivate;
 
@@ -97,25 +102,12 @@ ti_wnd_channel_dispose (GObject *object)
     TIWndChannel *wnd_channel = TI_WND_CHANNEL (object);
     TIWndChannelPrivate *priv = TI_WND_CHANNEL_GET_PRIVATE (wnd_channel);
 
-    if (priv->disposed)
-    {
-        return;
-    }
-    else
-    {
-        priv->disposed = TRUE;
-    }
-
-    if (priv->handle_mapper != NULL)
-    {
-        g_object_unref (priv->handle_mapper);
-        priv->handle_mapper = NULL;
-    }
+    TI_OBJ_UNREF (priv->handle_mapper);
 
     if (priv->preferences != NULL)
     {
         g_signal_handlers_disconnect_by_func (priv->preferences,
-                                              G_CALLBACK (_ti_wnd_channel_contact_handle_display_mode_changed),
+                                              G_CALLBACK (_ti_wnd_channel_handle_display_mode_changed),
                                               wnd_channel);
 
         g_object_unref (priv->preferences);
@@ -127,29 +119,12 @@ ti_wnd_channel_dispose (GObject *object)
         gtk_widget_hide_all (priv->window);
     }
 
-    if (priv->glade_xml != NULL)
-    {
-        g_object_unref (priv->glade_xml);
-        priv->glade_xml = NULL;
-    }
-
-    if (priv->channel != NULL)
-    {
-        g_object_unref (priv->channel);
-        priv->channel = NULL;
-    }
-    
-    if (priv->page_group != NULL)
-    {
-        g_object_unref (priv->page_group);
-        priv->page_group = NULL;
-    }
-    
-    if (priv->page_text != NULL)
-    {
-        g_object_unref (priv->page_text);
-        priv->page_text = NULL;
-    }
+    TI_OBJ_UNREF (priv->glade_xml);
+    TI_OBJ_UNREF (priv->channel);
+    TI_OBJ_UNREF (priv->page_group);
+    TI_OBJ_UNREF (priv->page_room_list);
+    TI_OBJ_UNREF (priv->page_text);
+    TI_OBJ_UNREF (priv->page_properties);
 
     G_OBJECT_CLASS (ti_wnd_channel_parent_class)->dispose (object);
 }
@@ -160,18 +135,18 @@ ti_wnd_channel_dispose (GObject *object)
 static void
 ti_wnd_channel_class_init (TIWndChannelClass *ti_wnd_channel_class)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_wnd_channel_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (ti_wnd_channel_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_wnd_channel_dispose;
-		
-	/* Add private */
-	g_type_class_add_private (ti_wnd_channel_class, sizeof (TIWndChannelPrivate));
+    /* override base object methods */ 
+    gobject_class->dispose = ti_wnd_channel_dispose;
+
+    /* Add private */
+    g_type_class_add_private (ti_wnd_channel_class, sizeof (TIWndChannelPrivate));
 
     /* Signals */
 
-    /* Emitted when the channel is closed. */
-    ti_wnd_channel_class->close_wnd_id = 
+    /* Emitted when the window is closed. */
+    ti_wnd_channel_class->close_wnd_id =
         g_signal_newv ("close-wnd",
                        G_TYPE_FROM_CLASS (ti_wnd_channel_class),
                        G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
@@ -192,7 +167,6 @@ ti_wnd_channel_init (TIWndChannel *ti_wnd_channel)
 {
     TIWndChannelPrivate *priv = TI_WND_CHANNEL_GET_PRIVATE (ti_wnd_channel);
 
-    priv->disposed = FALSE;
     priv->self_handle_type = 0;
     priv->self_handle_number = 0;
     priv->handle_mapper = NULL;
@@ -203,7 +177,10 @@ ti_wnd_channel_init (TIWndChannel *ti_wnd_channel)
     priv->button_close = NULL;
     priv->channel = NULL;
     priv->page_group = NULL;
+    priv->page_room_list = NULL;
     priv->page_text = NULL;
+    priv->page_properties = NULL;
+    priv->should_close_window = FALSE;
 }
 
 /**
@@ -219,7 +196,7 @@ ti_wnd_channel_new (GtkWindow* parent, TIChannel* channel, TIHandleMapper* handl
     g_assert (channel != NULL);
     g_assert (parent != NULL);
 
-	wnd_channel = g_object_new (TI_TYPE_WND_CHANNEL, NULL);
+    wnd_channel = g_object_new (TI_TYPE_WND_CHANNEL, NULL);
 
     priv = TI_WND_CHANNEL_GET_PRIVATE (wnd_channel);
     priv->parent = parent;
@@ -247,14 +224,14 @@ ti_wnd_channel_new (GtkWindow* parent, TIChannel* channel, TIHandleMapper* handl
     priv->notebook = GTK_NOTEBOOK (glade_xml_get_widget (priv->glade_xml, "notebook"));
     g_assert (GTK_IS_NOTEBOOK (priv->notebook));
 
-    g_signal_connect_swapped (priv->channel, "closed", G_CALLBACK (_ti_wnd_channel_channel_closed), wnd_channel);
+    g_signal_connect_swapped (priv->channel, "closed", G_CALLBACK (_ti_wnd_channel_close_window), wnd_channel);
 
     g_signal_connect_swapped (priv->window, "delete-event", G_CALLBACK (on_delete_window), wnd_channel);
 
     gtk_window_set_transient_for (GTK_WINDOW (priv->window), GTK_WINDOW (priv->parent));
 
-    g_signal_connect_swapped (priv->preferences, "contact-handle-display-changed",
-                              G_CALLBACK (_ti_wnd_channel_contact_handle_display_mode_changed), wnd_channel);
+    g_signal_connect_swapped (priv->preferences, "handle-display-mode-changed",
+                              G_CALLBACK (_ti_wnd_channel_handle_display_mode_changed), wnd_channel);
 
     _ti_wnd_channel_setup_first_page (wnd_channel);
     _ti_wnd_channel_setup_type_interface_page (wnd_channel);
@@ -297,7 +274,7 @@ _ti_wnd_channel_setup_first_page (TIWndChannel* wnd_channel)
 
     // Set handle type and number labels
     ti_channel_get_handle (priv->channel, &(priv->self_handle_type), &(priv->self_handle_number), &error);
-    
+
     label = GTK_LABEL (glade_xml_get_widget (priv->glade_xml, "label_handle_type_value"));
     if (error == NULL)
     {
@@ -312,15 +289,15 @@ _ti_wnd_channel_setup_first_page (TIWndChannel* wnd_channel)
 
     priv->label_handle_number = GTK_LABEL (glade_xml_get_widget (priv->glade_xml, "label_handle_number"));
     g_assert (priv->label_handle_number != NULL && GTK_IS_LABEL (priv->label_handle_number));
-    
+
     priv->label_handle_number_value = GTK_LABEL (glade_xml_get_widget (priv->glade_xml, "label_handle_number_value"));
     g_assert (priv->label_handle_number_value != NULL && GTK_IS_LABEL (priv->label_handle_number_value));
 
     if (error == NULL)
     {
         if (priv->self_handle_type != TI_CONNECTION_HANDLE_TYPE_CONTACT ||
-            ti_preferences_get_contact_handle_display_mode (priv->preferences) == 
-                TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+            ti_preferences_get_handle_display_mode (priv->preferences) ==
+                TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
         {
             str = g_strdup_printf ("%u", priv->self_handle_number);
             gtk_label_set_text (priv->label_handle_number_value, str);
@@ -338,7 +315,6 @@ _ti_wnd_channel_setup_first_page (TIWndChannel* wnd_channel)
         gtk_label_set_text (priv->label_handle_number_value, error->message);
     }
 
-    
     // Interfaces List
     priv->interfaces_list = gtk_list_store_new (1, G_TYPE_STRING);
     _ti_wnd_channel_build_interfaces_treeview (wnd_channel);
@@ -363,10 +339,11 @@ _ti_wnd_channel_close_channel (TIWndChannel* wnd_channel)
     if (error != NULL)
     {
         // TODO: Display a message or something.
-        
-        // Force the window to close.
-        _ti_wnd_channel_channel_closed (wnd_channel);
+        return;
     }
+
+    // Force the window to close.
+    _ti_wnd_channel_close_window (wnd_channel);
 }
 
 /**
@@ -440,9 +417,9 @@ ti_wnd_channel_show (TIWndChannel* wnd_channel)
 static gboolean
 on_delete_window (TIWndChannel* wnd_channel, GdkEvent  *event)
 {
-    g_debug ("ti_wnd_channel->on_delete_window()");
-
-    _ti_wnd_channel_close_channel (wnd_channel);
+    // Just close the window. If the user really wants to close the *telepathy channel*
+    // he should click the "Close" *button* instead.
+    _ti_wnd_channel_close_window (wnd_channel);
 
     return TRUE;
 }
@@ -464,7 +441,7 @@ _ti_wnd_channel_add_interface_pages (TIWndChannel* wnd_channel)
     {
         _ti_wnd_channel_add_interface_page (wnd_channel, interfaces[i]);
     }
-    
+
     g_strfreev (interfaces);
 }
 
@@ -475,15 +452,26 @@ static void
 _ti_wnd_channel_add_interface_page (TIWndChannel* wnd_channel, gchar* interface_name)
 {
     TIWndChannelPrivate *priv = TI_WND_CHANNEL_GET_PRIVATE (wnd_channel);
-    
+    TIProperties* properties = NULL;
+
     if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Channel.Interface.Group"))
     {
         priv->page_group = ti_page_channel_group_new (priv->notebook, GTK_WINDOW (priv->window), priv->channel, priv->handle_mapper);
     }
+    else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Properties"))
+    {
+        properties = ti_properties_new (ti_channel_get_service_name (priv->channel),
+                                        ti_channel_get_object_path (priv->channel));
+
+        priv->page_properties = ti_page_properties_new (priv->notebook, properties);
+    }
     else
     {
         g_warning ("Unrecognized interface: \"%s\"", interface_name);
     }
+
+    // Clean up
+    TI_OBJ_UNREF (properties);
 }
 
 /**
@@ -497,11 +485,11 @@ _get_handle_type_str (guint handle_type)
         case 0:
             return g_strdup ("0 - CONNECTION_HANDLE_TYPE_NONE");
             break;
-            
+
         case 1:
             return g_strdup ("1 - CONNECTION_HANDLE_TYPE_CONTACT");
             break;
-        
+
         case 2:
             return g_strdup ("2 - CONNECTION_HANDLE_TYPE_ROOM");
             break;
@@ -509,20 +497,21 @@ _get_handle_type_str (guint handle_type)
         case 3:
             return g_strdup ("3 - CONNECTION_HANDLE_TYPE_LIST");
             break;
-            
+
         default:
             return g_strdup_printf ("%u - Unknown type", handle_type);
     }
 }
 
 /**
- * Channel Closed
+ * Window Closed
  */
 static void
-_ti_wnd_channel_channel_closed (TIWndChannel* wnd_channel)
+_ti_wnd_channel_close_window (TIWndChannel* wnd_channel)
 {
     TIWndChannelPrivate *priv = TI_WND_CHANNEL_GET_PRIVATE (wnd_channel);
 
+    // Make the connection window destroy us.
     g_signal_emit (wnd_channel, TI_WND_CHANNEL_GET_CLASS(wnd_channel)->close_wnd_id, 0, NULL);
     gtk_widget_hide_all (priv->window);
 }
@@ -546,6 +535,10 @@ _ti_wnd_channel_setup_type_interface_page (TIWndChannel* wnd_channel)
     {
         priv->page_text = ti_page_channel_text_new (priv->notebook, priv->channel);
     }
+    else if (g_str_equal (str, "org.freedesktop.Telepathy.Channel.Type.RoomList"))
+    {
+        priv->page_room_list = ti_page_room_list_new (priv->notebook, priv->channel, priv->handle_mapper);
+    }
 
     CLEAN_UP:
 
@@ -556,10 +549,10 @@ _ti_wnd_channel_setup_type_interface_page (TIWndChannel* wnd_channel)
 }
 
 /**
- * Contact Handle Display Mode Changed
+ * Handle Display Mode Changed
  */
 static void
-_ti_wnd_channel_contact_handle_display_mode_changed (TIWndChannel* wnd_channel, guint contact_handle_display_mode)
+_ti_wnd_channel_handle_display_mode_changed (TIWndChannel* wnd_channel, guint handle_display_mode)
 {
     TIWndChannelPrivate* priv = TI_WND_CHANNEL_GET_PRIVATE (wnd_channel);
     gchar* str = NULL;
@@ -567,7 +560,7 @@ _ti_wnd_channel_contact_handle_display_mode_changed (TIWndChannel* wnd_channel,
     if (priv->self_handle_type != TI_CONNECTION_HANDLE_TYPE_CONTACT)
         return;
 
-    if (contact_handle_display_mode == TI_PREFERENCES_CONTACT_HANDLE_DISPLAY_HANDLE)
+    if (handle_display_mode == TI_PREFERENCES_HANDLE_DISPLAY_HANDLE)
     {
         gtk_label_set_text (priv->label_handle_number, "Number:");
         str = g_strdup_printf ("%u", priv->self_handle_number);
