@@ -5,7 +5,7 @@
  * ti-wnd-connection.c:
  * Connection Window - UI to expose org.freedesktop.Telepathy.Connection functionality
  * 
- * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
+ * Copyright (C) 2006, 2007 INdT - Instituto Nokia de Tecnologia
  * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,10 @@
 #include "ti-dlg-request-channel.h"
 #include "ti-wnd-channel.h"
 #include "ti-page-aliasing.h"
+#include "ti-page-capabilities.h"
 #include "ti-page-presence.h"
+#include "ti-page-properties.h"
+#include "ti-properties.h"
 #include "ti-constants.h"
 #include "ti-config.h"
 #include "ti-handle-mapper.h"
@@ -52,6 +55,7 @@ static void _ti_wnd_connection_build_channels_treeview (TIWndConnection* wnd_con
 static void _ti_wnd_connection_connect (TIWndConnection* wnd_connection);
 static void _ti_wnd_connection_disconnect (TIWndConnection* wnd_connection);
 static void _ti_wnd_connection_on_conn_status_changed (TIWndConnection* wnd_connection, guint status, guint reason);
+static void _ti_wnd_connection_on_new_channel (TIWndConnection* wnd_connection, TIChannel* channel, gboolean suppress_handler);
 static void _ti_wnd_connection_refresh_channels_list (TIWndConnection* wnd_connection);
 static void _ti_wnd_connection_request_channel (TIWndConnection* wnd_connection);
 static void _ti_wnd_connection_request_handles (TIWndConnection* wnd_connection);
@@ -77,8 +81,6 @@ static void _ti_wnd_connection_on_connected (TIWndConnection* wnd_connection);
  * Instance private data.
  */
 struct _TIWndConnectionPrivate {
-    gboolean disposed;
-
     TIHandleMapper* handle_mapper;
 
     TIDlgHoldHandles* dlg_hold_handles;
@@ -103,7 +105,7 @@ struct _TIWndConnectionPrivate {
     GtkWidget* button_disconnect;
 
     GtkWidget* button_request_handles;
-    
+
     GtkWidget* button_request_channel;
     GtkWidget* button_refresh_channels_list;
 
@@ -111,12 +113,14 @@ struct _TIWndConnectionPrivate {
     GtkWidget* button_inspect_handles;
     GtkWidget* button_release_handles;
     GtkWidget* button_hold_handles;
-    
+
     /* An array of TIWndChannel* containing all channel windows currently open. */
     GPtrArray* channel_wnds;
 
     TIPageAliasing* page_aliasing;
+    TIPageCapabilities* page_capabilities;
     TIPagePresence* page_presence;
+    TIPageProperties* page_properties;
 };
 typedef struct _TIWndConnectionPrivate TIWndConnectionPrivate;
 
@@ -133,51 +137,26 @@ ti_wnd_connection_dispose (GObject *object)
     guint i;
     GObject* obj;
 
-    if (priv->disposed)
-    {
-        return;
-    }
-    else
-    {
-        priv->disposed = TRUE;
-    }
-
-    if (priv->handle_mapper != NULL)
-    {
-        g_object_unref (priv->handle_mapper);
-        priv->handle_mapper = NULL;
-    }
-
-    if (priv->dlg_hold_handles != NULL)
-    {
-        g_object_unref (priv->dlg_hold_handles);
-        priv->dlg_hold_handles = NULL;
-    }
-
-    if (priv->dlg_get_handles != NULL)
-    {
-        g_object_unref (priv->dlg_get_handles);
-        priv->dlg_get_handles = NULL;
-    }
-    
-    if (priv->dlg_request_channel != NULL)
-    {
-        g_object_unref (priv->dlg_request_channel);
-        priv->dlg_request_channel = NULL;
-    }
-
-    if (priv->glade_xml != NULL)
-    {
-        g_object_unref (priv->glade_xml);
-        priv->glade_xml = NULL;
-    }
+    TI_OBJ_UNREF (priv->handle_mapper);
+    TI_OBJ_UNREF (priv->dlg_hold_handles);
+    TI_OBJ_UNREF (priv->dlg_get_handles);
+    TI_OBJ_UNREF (priv->dlg_request_channel);
+    TI_OBJ_UNREF (priv->glade_xml);
 
     if (priv->connection != NULL)
     {
+        g_signal_handlers_disconnect_by_func (priv->connection,
+                                              G_CALLBACK (_ti_wnd_connection_on_conn_status_changed),
+                                              wnd_connection);
+
+        g_signal_handlers_disconnect_by_func (priv->connection,
+                                              G_CALLBACK (_ti_wnd_connection_on_new_channel),
+                                              wnd_connection);
+
         g_object_unref (priv->connection);
         priv->connection = NULL;
     }
-    
+
     if (priv->channel_wnds != NULL)
     {
         for (i = 0; i < priv->channel_wnds->len; i++)
@@ -189,17 +168,10 @@ ti_wnd_connection_dispose (GObject *object)
         priv->channel_wnds = NULL;
     }
 
-    if (priv->page_aliasing != NULL)
-    {
-        g_object_unref (priv->page_aliasing);
-        priv->page_aliasing = NULL;
-    }
-
-    if (priv->page_presence != NULL)
-    {
-        g_object_unref (priv->page_presence);
-        priv->page_presence = NULL;
-    }
+    TI_OBJ_UNREF (priv->page_aliasing);
+    TI_OBJ_UNREF (priv->page_capabilities);
+    TI_OBJ_UNREF (priv->page_presence);
+    TI_OBJ_UNREF (priv->page_properties);
 
     G_OBJECT_CLASS (ti_wnd_connection_parent_class)->dispose (object);
 }
@@ -210,13 +182,13 @@ ti_wnd_connection_dispose (GObject *object)
 static void
 ti_wnd_connection_class_init (TIWndConnectionClass *ti_wnd_connection_class)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_wnd_connection_class);
+    GObjectClass *gobject_class = G_OBJECT_CLASS (ti_wnd_connection_class);
 
-	/* override base object methods */ 
-	gobject_class->dispose = ti_wnd_connection_dispose;
-		
-	/* Add private */
-	g_type_class_add_private (ti_wnd_connection_class, sizeof (TIWndConnectionPrivate));
+    /* override base object methods */
+    gobject_class->dispose = ti_wnd_connection_dispose;
+
+    /* Add private */
+    g_type_class_add_private (ti_wnd_connection_class, sizeof (TIWndConnectionPrivate));
 
     /* Signals */
 
@@ -242,7 +214,6 @@ ti_wnd_connection_init (TIWndConnection *ti_wnd_connection)
 {
     TIWndConnectionPrivate *priv = TI_WND_CONNECTION_GET_PRIVATE (ti_wnd_connection);
 
-    priv->disposed = FALSE;
     priv->handle_mapper = NULL;
     priv->dlg_get_handles = NULL;
     priv->dlg_request_channel = NULL;
@@ -252,7 +223,9 @@ ti_wnd_connection_init (TIWndConnection *ti_wnd_connection)
     priv->notebook = NULL;
     priv->channel_wnds = g_ptr_array_new ();
     priv->page_aliasing = NULL;
+    priv->page_capabilities = NULL;
     priv->page_presence = NULL;
+    priv->page_properties = NULL;
 }
 
 /**
@@ -296,7 +269,7 @@ ti_wnd_connection_new_with_conn (GtkWindow* parent, TIConnection* connection)
     g_assert (connection != NULL);
     g_assert (parent != NULL);
 
-	wnd_connection = g_object_new (TI_TYPE_WND_CONNECTION, NULL);
+    wnd_connection = g_object_new (TI_TYPE_WND_CONNECTION, NULL);
 
     priv = TI_WND_CONNECTION_GET_PRIVATE (wnd_connection);
     priv->parent = parent;
@@ -305,6 +278,7 @@ ti_wnd_connection_new_with_conn (GtkWindow* parent, TIConnection* connection)
     g_object_ref (connection);
 
     g_signal_connect_swapped (connection, "status-changed", G_CALLBACK (_ti_wnd_connection_on_conn_status_changed), wnd_connection);
+    g_signal_connect_swapped (connection, "new-channel", G_CALLBACK (_ti_wnd_connection_on_new_channel), wnd_connection);
 
     glade_file_path = g_strdup_printf ("%s%s", TI_DATA_DIR_PREFIX, "wnd-connection.xml");
     priv->glade_xml = glade_xml_new (glade_file_path, NULL, NULL);
@@ -378,7 +352,7 @@ _ti_wnd_connection_setup_first_page (TIWndConnection* wnd_connection)
     // Interfaces List
     priv->interfaces_list = gtk_list_store_new (1, G_TYPE_STRING);
     _ti_wnd_connection_build_interfaces_treeview (wnd_connection);
-    
+
     // Channels List
     priv->channels_list = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
     _ti_wnd_connection_build_channels_treeview (wnd_connection);
@@ -547,21 +521,21 @@ _ti_wnd_connection_build_channels_treeview (TIWndConnection* wnd_connection)
                                                        "text", 0,
                                                        NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_channels), column);
-    
+
     /* Type column */
     column = gtk_tree_view_column_new_with_attributes ("Type",
                                                        renderer,
                                                        "text", 1,
                                                        NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_channels), column);
-    
+
     /* Handle type column */
     column = gtk_tree_view_column_new_with_attributes ("H. Type",
                                                        renderer,
                                                        "text", 2,
                                                        NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_channels), column);
-    
+
     /* Handle number column */
     column = gtk_tree_view_column_new_with_attributes ("H. Number",
                                                        renderer,
@@ -615,8 +589,6 @@ on_delete_window (TIWndConnection* wnd_connection, GdkEvent  *event)
 {
     TIWndConnectionPrivate *priv = TI_WND_CONNECTION_GET_PRIVATE (wnd_connection);
 
-    g_debug ("ti_wnd_connection->on_delete_window()");
-
     gtk_widget_hide_all (priv->window);
 
     g_signal_emit (wnd_connection, TI_WND_CONNECTION_GET_CLASS(wnd_connection)->close_signal_id, 0, NULL);
@@ -624,13 +596,6 @@ on_delete_window (TIWndConnection* wnd_connection, GdkEvent  *event)
     return TRUE;
 }
 
-/**
- * Create Interface Pages
- * Creates a notebook page for every interface (org.freedesktop.Telepathy.Connection.Interface.*)
- * implemented by this connection.
- */
-
-
 /**
  * Connect
  */
@@ -787,6 +752,47 @@ _ti_wnd_connection_on_conn_status_changed (TIWndConnection* wnd_connection, guin
     }
 }
 
+/**
+ * On New Channel
+ */
+static void
+_ti_wnd_connection_on_new_channel (TIWndConnection* wnd_connection, TIChannel* channel, gboolean suppress_handler)
+{
+    TIWndConnectionPrivate* priv = TI_WND_CONNECTION_GET_PRIVATE (wnd_connection);
+    TIWndChannel* channel_window = NULL;
+    GError* error = NULL;
+    gchar* channel_type = NULL;
+
+    channel_type = ti_channel_get_channel_type (channel, &error);
+    if (error != NULL)
+    {
+        // TODO: Display a message or something.
+        goto CLEAN_UP;
+    }
+
+    // We don't popup channel windows for suppressed and/or ContactList channels.
+    if (!suppress_handler && strcmp (channel_type, "org.freedesktop.Telepathy.Channel.Type.ContactList") != 0)
+    {
+        channel_window = ti_wnd_channel_new (GTK_WINDOW (priv->window), channel, priv->handle_mapper);
+        if (channel_window != NULL)
+        {
+            g_ptr_array_add (priv->channel_wnds, channel_window);
+            g_signal_connect_swapped (channel_window, "close-wnd", G_CALLBACK (_ti_wnd_connection_closed_channel_window), wnd_connection);
+            ti_wnd_channel_show (channel_window);
+        }
+    }
+
+    _ti_wnd_connection_refresh_channels_list (wnd_connection);
+
+    CLEAN_UP:
+
+    if (error != NULL)
+        g_error_free (error);
+
+    if (channel_type != NULL)
+        g_free (channel_type);
+}
+
 /**
  * Refresh Channels List
  */
@@ -908,13 +914,13 @@ _ti_wnd_connection_request_channel (TIWndConnection* wnd_connection)
     GError* error = NULL;
     TIChannel* channel;
     TIWndChannel* channel_window;
-    
+
     ok = ti_dlg_request_channel_run (priv->dlg_request_channel, &type, &handle_type, &handle, &suppress_handler);
     if (!ok)
     {
         goto CLEAN_UP;
     }
-    
+
     channel = ti_connection_request_channel (priv->connection, type, handle_type, handle, suppress_handler, &error);
     if (error != NULL || channel == NULL)
     {
@@ -1048,12 +1054,12 @@ _ti_wnd_connection_closed_channel_window (TIWndConnection* wnd_connection, TIWnd
     TIWndConnectionPrivate *priv = TI_WND_CONNECTION_GET_PRIVATE (wnd_connection);
     gboolean ok;
 
-    g_debug ("_ti_wnd_connection_closed_channel_window()");
-
     ok = g_ptr_array_remove (priv->channel_wnds, wnd_channel);
     g_assert (ok);
 
     g_object_unref (wnd_channel);
+
+    _ti_wnd_connection_refresh_channels_list (wnd_connection);
 }
 
 /**
@@ -1338,10 +1344,20 @@ _ti_wnd_connection_remove_interface_pages (TIWndConnection* wnd_connection)
         ti_page_detach_from_notebook (TI_PAGE (priv->page_aliasing));
     }
 
+    if (priv->page_capabilities != NULL)
+    {
+        ti_page_detach_from_notebook (TI_PAGE (priv->page_capabilities));
+    }
+
     if (priv->page_presence != NULL)
     {
         ti_page_detach_from_notebook (TI_PAGE (priv->page_presence));
     }
+
+    if (priv->page_properties != NULL)
+    {
+        ti_page_detach_from_notebook (TI_PAGE (priv->page_properties));
+    }
 }
 
 /**
@@ -1363,7 +1379,7 @@ _ti_wnd_connection_create_interface_pages (TIWndConnection* wnd_connection)
     {
         _ti_wnd_connection_create_interface_page (wnd_connection, interfaces[i]);
     }
-    
+
     g_strfreev (interfaces);
 }
 
@@ -1374,7 +1390,8 @@ static void
 _ti_wnd_connection_create_interface_page (TIWndConnection* wnd_connection, gchar* interface_name)
 {
     TIWndConnectionPrivate *priv = TI_WND_CONNECTION_GET_PRIVATE (wnd_connection);
-    
+    TIProperties* properties = NULL;
+
     if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.Presence"))
     {
         if (priv->page_presence == NULL)
@@ -1397,9 +1414,37 @@ _ti_wnd_connection_create_interface_page (TIWndConnection* wnd_connection, gchar
             ti_page_attach_to_notebook (TI_PAGE (priv->page_aliasing), priv->notebook);
         }
     }
+    else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Connection.Interface.Capabilities"))
+    {
+        if (priv->page_capabilities == NULL)
+        {
+            priv->page_capabilities = ti_page_capabilities_new (GTK_WINDOW (priv->window), priv->notebook, priv->connection, priv->handle_mapper);
+        }
+        else
+        {
+            ti_page_attach_to_notebook (TI_PAGE (priv->page_capabilities), priv->notebook);
+        }
+    }
+    else if (g_str_equal (interface_name, "org.freedesktop.Telepathy.Properties"))
+    {
+        if (priv->page_properties == NULL)
+        {
+            properties = ti_properties_new (ti_connection_get_service_name (priv->connection),
+                                            ti_connection_get_object_path (priv->connection));
+
+            priv->page_properties = ti_page_properties_new (priv->notebook, properties);
+        }
+        else
+        {
+            ti_page_attach_to_notebook (TI_PAGE (priv->page_properties), priv->notebook);
+        }
+    }
     else
     {
         // TODO: Add more interface pages if that's the case.
         g_warning ("Unrecognized interface: \"%s\"", interface_name);
     }
+
+    // Clean up
+    TI_OBJ_UNREF (properties);
 }
