@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <gdk/gdkkeysyms.h>
 
 struct _TIWndMainClass {
     GObjectClass parent;
@@ -52,6 +53,7 @@ static void _ti_wnd_main_fill_protocols_list (TIWndMain* wnd_main);
 static void _ti_wnd_main_build_prot_params_treeview (TIWndMain* wnd_main);
 static void _ti_wnd_main_fill_prot_params_list (TIWndMain* wnd_main, const gchar* protocol_name);
 static void _ti_wnd_main_conn_manager_entry_changed (TIWndMain* wnd_main);
+static gboolean _ti_wnd_main_conn_manager_entry_key_pressed (TIWndMain* self, GdkEventKey* event);
 static void _ti_wnd_main_request_connection (TIWndMain* wnd_main);
 static gchar* _ti_wnd_main_get_selected_protocol_name (TIWndMain* wnd_main);
 static void _ti_wnd_main_on_conn_window_closed (TIWndMain* wnd_main, TIWndConnection* wnd_connection);
@@ -62,6 +64,7 @@ static void _ti_wnd_main_show_about_dialog (TIWndMain* wnd_main);
 static void _ti_wnd_main_show_preferences_dialog (TIWndMain* wnd_main);
 static void _ti_wnd_main_find_installed_conn_managers (TIWndMain* self);
 
+
 /**
  * Instance private data.
  */
@@ -268,7 +271,6 @@ _ti_wnd_main_setup_conn_man_page (TIWndMain* wnd_main)
 {
     TIWndMainPrivate *priv = TI_WND_MAIN_GET_PRIVATE (wnd_main);
     GtkWidget* widget = NULL;
-    GtkCellRenderer* renderer = NULL;
 
     // Setup page label
     widget = gtk_notebook_get_nth_page (priv->notebook, 0);
@@ -284,12 +286,11 @@ _ti_wnd_main_setup_conn_man_page (TIWndMain* wnd_main)
     g_assert (GTK_IS_COMBO_BOX_ENTRY (priv->comboentry_conn_man));
     g_signal_connect_swapped (GTK_COMBO_BOX (priv->comboentry_conn_man), "changed", G_CALLBACK (_ti_wnd_main_conn_manager_entry_changed), wnd_main);
 
+    widget = gtk_bin_get_child (GTK_BIN (priv->comboentry_conn_man));
+    g_assert (widget != NULL && GTK_IS_ENTRY (widget));
+    g_signal_connect_swapped (widget, "key-press-event", G_CALLBACK (_ti_wnd_main_conn_manager_entry_key_pressed), wnd_main);
+
     priv->conn_managers_list = gtk_list_store_new (1, G_TYPE_STRING);
-    renderer = gtk_cell_renderer_text_new ();
-    gtk_combo_box_set_model (GTK_COMBO_BOX (priv->comboentry_conn_man), NULL);
-    gtk_cell_layout_clear (GTK_CELL_LAYOUT (priv->comboentry_conn_man));
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->comboentry_conn_man), renderer, FALSE);
-    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->comboentry_conn_man), renderer, "text", 0);
     gtk_combo_box_set_model (GTK_COMBO_BOX (priv->comboentry_conn_man), GTK_TREE_MODEL (priv->conn_managers_list) );
     gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (priv->comboentry_conn_man), 0);
 
@@ -517,6 +518,23 @@ _ti_wnd_main_conn_manager_entry_changed (TIWndMain* wnd_main)
     g_free (active_text);
 }
 
+/**
+ * Connection Manager Entry - Key Pressed
+ * Called when the connection manager entry has focus and a key is pressed.
+ */
+static gboolean
+_ti_wnd_main_conn_manager_entry_key_pressed (TIWndMain* self, GdkEventKey* event)
+{
+    if (event->keyval == GDK_Return && event->type == GDK_KEY_PRESS)
+    {
+        // The user pressed <Enter>, therefore we should query the
+        // connection manager whose name he just wrote.
+        _ti_wnd_main_query_conn_man (self);
+    }
+
+    return FALSE; // propagate the event further
+}
+
 /**
  * Request Connection
  */
